@@ -18,6 +18,7 @@ from typing import (
     ClassVar,
     Dict,
     Generic,
+    Iterable,
     Iterator,
     List,
     Optional,
@@ -450,16 +451,30 @@ class RGBDImage(CompositeImageBase):
         assert False
 
 
-class ElementDict(Dict[Type[ElementBase], ElementBase]):
+class ElementDict(Dict[Type[PrimitiveElementBase], PrimitiveElementBase]):
     def __init__(self, elems: Sequence[ElementBase]):
         for elem in elems:
-            self[elem.__class__] = elem
-        assert_equal_with_message(len(set(self.keys())), len(elems), "num of element")
+            self.__setitem__(elem.__class__, elem)
+
+    def get_subdict(self, keys: Iterable[Type[ElementBase]]) -> "ElementDict":
+        return ElementDict([self.__getitem__(key) for key in keys])
+
+    def __setitem__(self, elem_type: Type[ElementBase], elem: ElementBase):
+        assert type(elem) == elem_type
+
+        if isinstance(elem, CompositeImageBase):
+            assert issubclass(elem_type, CompositeImageBase)
+            for sub_elem_type, sub_elem in zip(elem.image_types, elem.images):
+                super().__setitem__(sub_elem_type, sub_elem)
+        elif isinstance(elem, PrimitiveElementBase):
+            assert issubclass(elem_type, PrimitiveElementBase)
+            super().__setitem__(elem_type, elem)
+        else:
+            assert False
 
     def __getitem__(self, key: Type[ElementT]) -> ElementT:
         if issubclass(key, PrimitiveElementBase):
             return super().__getitem__(key)  # type: ignore
-
         elif issubclass(key, CompositeImageBase):
             if key in self:
                 return super().__getitem__(key)  # type: ignore
@@ -667,7 +682,7 @@ class EpisodeData(HasTypeShapeTable, Hashable):
             sequence_list: list of ElementSequence to construct the episode
             timestamp_seq: optional timestamp information
             metadata: optional metadata
-            check_terminate_flag: if True, validation of TerminateFlag sequence will be  conducted.                 (see: validate_terminate_flags function)
+            check_terminate_flag: if True, validation of TerminateFlag sequence will be  conducted. (see: validate_terminate_flags function)
         """
 
         if metadata is None:
@@ -710,13 +725,13 @@ class EpisodeData(HasTypeShapeTable, Hashable):
             edict_list: list of ElementDict to construct the episode
             timestamp_seq: optional timestamp information
             metadata: optional metadata
-            check_terminate_flag: if True, validation of TerminateFlag sequence will be  conducted.                 (see: validate_terminate_flags function)
+            check_terminate_flag: if True, validation of TerminateFlag sequence will be  conducted. (see: validate_terminate_flags function)
         """
 
         # all edict must have the same keys
         key_set_ref = set(edict_list[0].keys())
         for edict in edict_list:
-            assert set(edict.keys()) == key_set_ref
+            assert_equal_with_message(set(edict.keys()), key_set_ref, "key set")
 
         elem_seq_list = []
         for key in key_set_ref:
